//////////////////////////////////////////////////////////////////////
// trainNet.java
// 2005.03.07 by Geof Fang
// Adapted from code by Aydin Gurel, 2003.
// aydingurel@hotmail.com
// http://aydingurel.brinkster.com/neural
//
// This class will train a basic feed-forward with back prop neural
// net and save the weights, and net config to a file, for future
// use.  It takes input from a file.
//
// Revision History:  2004.03.07 initial write
//////////////////////////////////////////////////////////////////////

import java.io.*;

public class trainNet
{
  static public void main( String args[] )
  {
    // Construct a Randomizer object using a seed
    Randomizer randomizer = new Randomizer(4);

    // Create the neural net architecture
    int[] noofneurons = {4, 6, 4, 1};
    double[] learnratecoeff = {1, 1, 1, 1};
    char[] axonfamily = {'t', 't', 't', 't'};
    double[] momentumrate = {1, .8, .7, .5};
    double[] flatness = {1, 1, 1, 1};
    
    // Save the net to disk
    System.out.println("Creating the net, and saving to disk...");
    NeuralNet myNet = new NeuralNet(noofneurons, learnratecoeff, axonfamily, momentumrate, flatness, randomizer);
    try
    {
      myNet.SaveConfig("heimdall.nnc");
    }
    catch( IOException e )
    {
      System.out.println( "An error occured while saving the configuration." );
    }

    // Create a pattern set with 4 input, and 1 output values.
    // Randomly choose 80% of data for training, 10% for cross validation, 10% for testing.
    PatternSet mypatterns = new PatternSet("heimdall_train.csv", 4, 1, .8, .1, .1, randomizer);

    // Display the current error ratio.
    System.out.println("\n\nError ratio before training: " + myNet.CrossValErrorRatio(mypatterns) );

    // Continue to train the net until the error ratio drops below 0.1
    while( myNet.CrossValErrorRatio( mypatterns ) > 0.1 )
    {
      myNet.BatchTrainPatterns(mypatterns.trainingpatterns, .8);
      System.out.println( "Training the net. Error ratio: " + myNet.CrossValErrorRatio( mypatterns ) );
    }

    // Check the error ratio using the test data
    System.out.println("Error ratio of the test data: " + myNet.TestErrorRatio(mypatterns) );

    // Inform the user that the training is over, and save the weights of the neural net.    
    System.out.println("Training is over");
    System.out.println("Saving the weights\n");
    
    try
    {
      myNet.SaveWeights( "heimdall.nnw" );
    }

    catch( IOException e )
    { }

    // clean up the objects
    mypatterns = null;
    myNet = null;
    randomizer = null;
  }
}
